Mario.Actor = function(x,y,data,s){
	this.data = data;

	this.pos = {x:x, y:y};
	this.speed = {x:0, y:0};

	this.state = s;
	this.action = Mario.Enum.ACTION_IDLE;
	this.facing = Mario.Enum.FACING_RIGHT;
};
Mario.Actor.create = function(x,y,w,h,s){
	return new Mario.Actor(x,y,w,h,s);
};
Mario.Actor.prototype = {
	pos: null,
	speed: null,
	frame: null,
	state: null, // small, big, fire
	action: null,

	data: null,
	
	render: function(frame){
		switch (this.action) {
			case Mario.Enum.ACTION_WALKING:
				this.frame = this.data.semantics[this.state].walk1 + Math.floor((frame % 6)/2);
				break
			case Mario.Enum.ACTION_AIRBORNE:
				this.frame = this.data.semantics[this.state].jump;
				break;
			case Mario.Enum.ACTION_IDLE:
				this.frame = this.data.semantics[this.state].idle;
				break;
			case Mario.Enum.ACTION_DUCKING:
				this.frame = this.data.semantics[this.state].duck;
				break;
			default:
 				console.log("unknown action", this.action);
		}
	},
	getHitbox: function(){
		// the hitbox for the actor is a sub area of the sprite
		var hitbox = this.data.hitbox;
		if (this.state == Mario.Enum.STATE_SMALL || this.action == Mario.Enum.ACTION_DUCKING) hitbox = hitbox.small;
		else hitbox = hitbox.big;

		var tw = Mario.Config.tileSize.w;
		var th = Mario.Config.tileSize.h;

		return {x:hitbox.x/tw, y:hitbox.y/th, w:hitbox.w/tw, h:hitbox.h/th};
	},
	updateSpeed: function(standing){
		// x will not change if already 0
		this.speed.x *= Mario.Config.friction;
		if (Math.abs(this.speed.x) < Mario.Config.stopThreshhold) {
			this.speed.x = 0;
			if (this.action != Mario.Enum.ACTION_AIRBORNE && this.action != Mario.Enum.ACTION_DUCKING) this.updateAction(Mario.Enum.ACTION_IDLE);
		}
		// basically, y always gets changed if you're not standing.
		if (!standing) this.speed.y = Math.min(this.speed.y + Mario.Config.gravity, 1.00); // max 1 (game always adds some gravity)
	},
	getNextPosition: function(){
		var targetx = this.pos.x + this.speed.x;
		var targety = this.pos.y + this.speed.y;
		return {x:targetx, y:targety};
	},
	updateAction: function(action){
		switch(action) {
			case Mario.Enum.ACTION_SMALL:
				this.state = Mario.Enum.STATE_SMALL;
				this.action = Mario.Enum.ACTION_IDLE;
				break;
			case Mario.Enum.ACTION_BIG:
				this.state = Mario.Enum.STATE_BIG;
				this.action = Mario.Enum.ACTION_IDLE;
				break;
			case Mario.Enum.ACTION_FIRE:
				this.state = Mario.Enum.STATE_FIRE;
				this.action = Mario.Enum.ACTION_IDLE;
				break;
			case Mario.Enum.ACTION_AIRBORNE:
				this.action = Mario.Enum.ACTION_AIRBORNE;
				break;
			case Mario.Enum.ACTION_IDLE:
				this.action = Mario.Enum.ACTION_IDLE;
				break;
			case Mario.Enum.ACTION_FACE_LEFT:
				this.facing = Mario.Enum.FACING_LEFT;
				if (this.action != Mario.Enum.ACTION_AIRBORNE && this.action != Mario.Enum.ACTION_DUCKING) this.action = Mario.Enum.ACTION_WALKING;
				break;
			case Mario.Enum.ACTION_FACE_RIGHT:
				this.facing = Mario.Enum.FACING_RIGHT;
				if (this.action != Mario.Enum.ACTION_AIRBORNE && this.action != Mario.Enum.ACTION_DUCKING) this.action = Mario.Enum.ACTION_WALKING;
				break;
			case Mario.Enum.ACTION_DUCKING:
				this.action = Mario.Enum.ACTION_DUCKING;
				break;
		} 
		
	}
};


Mario.Actor.p1 = {
	// sprite
	width: 16,
	height: 32,
	// visual
	hitbox: {
		small: 	{
			x:  1,
			y: 16,
			w: 14,
			h: 16
		}, 
		big: {
			x:  0,
			y:  1,
			w: 16,
			h: 31
		}
	},
	semantics: {
		dead: Mario.Enum.FRAME_SMALL_DEAD,
		middle: Mario.Enum.FRAME_MEDIUM,
		small: {
			idle: Mario.Enum.FRAME_SMALL,
			walk1: Mario.Enum.FRAME_SMALL_WALK1,
			walk2: Mario.Enum.FRAME_SMALL_WALK2,
			walk3: Mario.Enum.FRAME_SMALL_WALK3,
			turn: Mario.Enum.FRAME_SMALL_TURN,
			jump: Mario.Enum.FRAME_SMALL_JUMP,
			duck: Mario.Enum.FRAME_SMALL, // ducking while small does not change the visual
			swim1: Mario.Enum.FRAME_SMALL_SWIM1,
			swim2: Mario.Enum.FRAME_SMALL_SWIM2,
			swim3: Mario.Enum.FRAME_SMALL_SWIM3,
			swim4: Mario.Enum.FRAME_SMALL_SWIM4,
			swim5: Mario.Enum.FRAME_SMALL_SWIM5,
			swim6: Mario.Enum.FRAME_SMALL_SWIM3 // large versions have 6 swim frames
		},
		big: {
			idle: Mario.Enum.FRAME_BIG,
			walk1: Mario.Enum.FRAME_BIG_WALK1,
			walk2: Mario.Enum.FRAME_BIG_WALK2,
			walk3: Mario.Enum.FRAME_BIG_WALK3,
			turn: Mario.Enum.FRAME_BIG_TURN,
			jump: Mario.Enum.FRAME_BIG_JUMP,
			duck: Mario.Enum.FRAME_BIG_DUCK,
			swim1: Mario.Enum.FRAME_BIG_SWIM1,
			swim2: Mario.Enum.FRAME_BIG_SWIM2,
			swim3: Mario.Enum.FRAME_BIG_SWIM3,
			swim4: Mario.Enum.FRAME_BIG_SWIM4,
			swim5: Mario.Enum.FRAME_BIG_SWIM5,
			swim6: Mario.Enum.FRAME_BIG_SWIM6
		},
		fire: {
			idle: Mario.Enum.FRAME_FIRE,
			walk1: Mario.Enum.FRAME_FIRE_WALK1,
			walk2: Mario.Enum.FRAME_FIRE_WALK2,
			walk3: Mario.Enum.FRAME_FIRE_WALK3,
			turn: Mario.Enum.FRAME_FIRE_TURN,
			jump: Mario.Enum.FRAME_FIRE_JUMP,
			duck: Mario.Enum.FRAME_FIRE_DUCK,
			swim1: Mario.Enum.FRAME_FIRE_SWIM1,
			swim2: Mario.Enum.FRAME_FIRE_SWIM2,
			swim3: Mario.Enum.FRAME_FIRE_SWIM3,
			swim4: Mario.Enum.FRAME_FIRE_SWIM4,
			swim5: Mario.Enum.FRAME_FIRE_SWIM5,
			swim6: Mario.Enum.FRAME_FIRE_SWIM6
		}
	},
	spriteMap: new function(){ var i = new Image; i.src = 'mario_sprites.png'; return i; } // simply returns an image object with src set
};
Mario.Actor.p2 = {
	// sprite
	width: 16,
	height: 32,
	// visual
	hitbox: {
		small: 	{
			x:  1,
			y: 16,
			w: 14,
			h: 16
		}, 
		big: {
			x:  0,
			y:  1,
			w: 16,
			h: 31
		}
	},
	semantics: {
		dead: Mario.Enum.FRAME_SMALL_DEAD,
		middle: Mario.Enum.FRAME_MEDIUM,
		small: {
			idle: Mario.Enum.FRAME_SMALL,
			walk1: Mario.Enum.FRAME_SMALL_WALK1,
			walk2: Mario.Enum.FRAME_SMALL_WALK2,
			walk3: Mario.Enum.FRAME_SMALL_WALK3,
			turn: Mario.Enum.FRAME_SMALL_TURN,
			jump: Mario.Enum.FRAME_SMALL_JUMP,
			duck: Mario.Enum.FRAME_SMALL, // ducking while small does not change the visual
			swim1: Mario.Enum.FRAME_SMALL_SWIM1,
			swim2: Mario.Enum.FRAME_SMALL_SWIM2,
			swim3: Mario.Enum.FRAME_SMALL_SWIM3,
			swim4: Mario.Enum.FRAME_SMALL_SWIM4,
			swim5: Mario.Enum.FRAME_SMALL_SWIM5,
			swim6: Mario.Enum.FRAME_SMALL_SWIM3 // large versions have 6 swim frames
		},
		big: {
			idle: Mario.Enum.FRAME_BIG,
			walk1: Mario.Enum.FRAME_BIG_WALK1,
			walk2: Mario.Enum.FRAME_BIG_WALK2,
			walk3: Mario.Enum.FRAME_BIG_WALK3,
			turn: Mario.Enum.FRAME_BIG_TURN,
			jump: Mario.Enum.FRAME_BIG_JUMP,
			duck: Mario.Enum.FRAME_BIG_DUCK,
			swim1: Mario.Enum.FRAME_BIG_SWIM1,
			swim2: Mario.Enum.FRAME_BIG_SWIM2,
			swim3: Mario.Enum.FRAME_BIG_SWIM3,
			swim4: Mario.Enum.FRAME_BIG_SWIM4,
			swim5: Mario.Enum.FRAME_BIG_SWIM5,
			swim6: Mario.Enum.FRAME_BIG_SWIM6
		},
		fire: {
			idle: Mario.Enum.FRAME_FIRE,
			walk1: Mario.Enum.FRAME_FIRE_WALK1,
			walk2: Mario.Enum.FRAME_FIRE_WALK2,
			walk3: Mario.Enum.FRAME_FIRE_WALK3,
			turn: Mario.Enum.FRAME_FIRE_TURN,
			jump: Mario.Enum.FRAME_FIRE_JUMP,
			duck: Mario.Enum.FRAME_FIRE_DUCK,
			swim1: Mario.Enum.FRAME_FIRE_SWIM1,
			swim2: Mario.Enum.FRAME_FIRE_SWIM2,
			swim3: Mario.Enum.FRAME_FIRE_SWIM3,
			swim4: Mario.Enum.FRAME_FIRE_SWIM4,
			swim5: Mario.Enum.FRAME_FIRE_SWIM5,
			swim6: Mario.Enum.FRAME_FIRE_SWIM6
		}
	},
	spriteMap: new function(){ var i = new Image; i.src = 'luigi_sprites.png'; return i; } // simply returns an image object with src set
};
Mario.Actor.enemy = {
	width: 16,
	height: 16,
	hitbox: {
		x: 1,
		y: 1,
		w: 14,
		h: 15
	},
	semantics: {
		walk1: 0,
		walk2: 1,
		dead: 2
	},
	spriteMap: new function(){ var i = new Image; i.src = 'enemy_sprites.png'; return i; } // simply returns an image object with src set
};

