Mario.Editor = function(){
	this.scene = new Mario.Scene(Mario.Enum.RUN_EDIT, this, new Mario.Input(this));

	this.initStamp();
	this.initTileMenu();
	this.initModes();
	
	document.body.onmousedown = this.dragStart.bind(this);
	document.body.onmousemove = this.dragging.bind(this);
	document.body.onmouseup = this.dragStop.bind(this);
	
	document.querySelector('.output input.exp').onclick = this.export.bind(this);
};
Mario.Editor.prototype = {
	currentMode: null,
	currentType: null,

	selectedTile: {clearing:true}, // {x,y,dragging,clearing}
	stamp: null,

	dragStart: function(e){
		this.selectedTile.dragging = true;
		this.dragging(e);
	},
	dragging: function(e){
		// mouse coords
		var x = e.clientX;
		var y = e.clientY;

		// target tile
		var targetx = Math.floor((this.scene.origin.x + x - this.scene.layers.container.offsetLeft) / Mario.Config.tileSize.w);
		var targety = Math.floor((this.scene.origin.y + y - this.scene.layers.container.offsetTop) / Mario.Config.tileSize.h);

		// if a tile is selected, update the stamp ghost image
		if (this.currentMode == Mario.Enum.MODE_TILE && !this.selectedTile.clearing) {
			this.updateStamp(x, y, targetx, targety);
		}
		
		// if dragging and inside canvas, paint
		if (this.selectedTile.dragging) {
			// dragging inside scene?
			if (targetx >= 0 && targety >= 0 && targetx < this.scene.map.tiles.w && targety < this.scene.map.tiles.h) {
				this.updateGrid(targetx, targety);
			}
		}
	},
	updateStamp: function(x,y, targetx, targety){
		// snap to grid here, if on grid
		var rx = x - this.scene.layers.container.offsetLeft;
		var ry = y - this.scene.layers.container.offsetTop;
		if (rx >= 0 && ry >= 0 && rx < this.scene.map.size.w && ry < this.scene.map.size.h) {
			x = this.scene.layers.container.offsetLeft + (targetx * Mario.Config.tileSize.w) - 1;
			y = this.scene.layers.container.offsetTop + (targety * Mario.Config.tileSize.h) - 1;
		} else {
			x -= 9;
			y -= 9;
		}

		this.stamp.style.left = x+'px';
		this.stamp.style.top = y+'px';		
	},
	updateGrid: function(targetx, targety){
		// "hash"
		var index = (targety*this.scene.map.tiles.w) + targetx;

		if (this.currentMode == Mario.Enum.MODE_TILE) {
			if (this.selectedTile.clearing) {
				this.scene.grids.background[index] = 0;
			} else {
				var x = Math.floor(this.selectedTile.x/this.scene.map.tileSet.size.w);
				var y = Math.floor(this.selectedTile.y/this.scene.map.tileSet.size.h);
				this.scene.grids.background[index] = (y * this.scene.map.tileSet.tiles.w) + x + 1;
			}
			this.scene.grids.background.redraw = true;
		} else if (this.currentMode == Mario.Enum.MODE_TYPE) {
			this.scene.grids.semantics[index] = this.currentType;
			this.scene.grids.semantics.redraw = true;
		}		
	},
	dragStop: function(e){
		if (e.target.className != 'tile selected') {
			this.selectedTile.dragging = false;
		}
	},

	export: function(){
		document.querySelector('.output input.bg').value = this.exp(this.scene.grids.background);
		document.querySelector('.output input.tp').value = this.exp(this.scene.grids.semantics);
	},
	exp: function(arr){
		var a = [];
		for (var i=0; i<arr.length; ++i) a[i] = arr[i] | 0;
		return a.join();
	},

	initTileMenu: function(){
		var div = document.querySelector('.tile-menu');
		div.onclick = this.menuClick.bind(this);
		// 13 and 11 are number of tiles wide and high of source image
		for (var j=0; j<this.scene.map.tileSet.tiles.h; ++j) {
			for (var i=0; i<this.scene.map.tileSet.tiles.w; ++i) {
				var tile = this.createThumb(i*(this.scene.map.tileSet.size.w+1), j*(this.scene.map.tileSet.size.h+1));
				div.appendChild(tile);
			}
		}
	},
	menuClick: function(e){
		if (e.target.className != 'tile-menu') {
			setTimeout(function(){
				if (e.target.className == 'tile') {
					if (this.selectedTile.thumb) {
						this.selectedTile.thumb.className = 'tile';
					}
					this.selectedTile = {
						x: e.target.getAttribute('data-x'), 
						y: e.target.getAttribute('data-y'),
						thumb: e.target,
						dragging: false
					};
					this.stamp.style.backgroundPosition = -this.selectedTile.x+'px -'+this.selectedTile.y+'px';
					this.stamp.style.display = 'block';
					e.target.className = 'tile selected';
				} else {
					e.target.className = 'tile';
					this.stamp.style.display = 'none';
					this.selectedTile = {clearing:true};
				}
			}.bind(this), 50);
		}
	},
	createThumb: function(x,y){
		var img = document.createElement('div');
		img.className = 'tile';
		img.style.backgroundPosition = -x+'px -'+y+'px';
		img.setAttribute('data-x', x);
		img.setAttribute('data-y', y);
		return img;
	},
	
	initModes: function(){
		this.currentMode = Mario.Enum.MODE_TILE;
		this.currentType = Mario.Enum.TYPE_REGULAR;
		
		var menu = document.querySelector('.mode-menu');
		var tiles = document.querySelector('.mode-tiles');
		var types = document.querySelector('.mode-types');
		
		menu.style.display = 'block';
		document.querySelector('.output').style.display = 'block';
		
		types.onclick = function(){
			this.currentMode = Mario.Enum.MODE_TYPE;
		}.bind(this);
		tiles.onclick = function(){
			this.currentMode = Mario.Enum.MODE_TILE;
		}.bind(this);
		menu.onclick = this.modeClick.bind(this);
	},
	modeClick: function(e){
		if (e.target.className.substring(0,4) == 'type') {
			var current = document.querySelector('.mode-menu .selected');
			if (e.target != current) {
				current.className = current.className.replace(/ selected/g,'');
				e.target.className += ' selected';
				current = e.target;
			}
			var type = current.className.split(' ')[1];
			switch (type) {
				case 'regular':
					this.currentType = Mario.Enum.TYPE_REGULAR;
					break;
				case 'block':
					this.currentType = Mario.Enum.TYPE_BLOCK;
					break;
				case 'deadly':
					this.currentType = Mario.Enum.TYPE_DEADLY;
					break;
				case 'water':
					this.currentType = Mario.Enum.TYPE_WATER;
					break;
				case 'slow':
					this.currentType = Mario.Enum.TYPE_SLOW;
					break;
				case 'fast':
					this.currentType = Mario.Enum.TYPE_FAST;
					break;
			}
		}
	},

	initStamp: function(){
		var stamp = this.stamp = document.createElement('div');
		stamp.className = 'tile stamp';
		stamp.style.opacity = 0.4;
		stamp.style.position = 'absolute';
		stamp.style.display = 'none'; // only show if a thumb is selected
		document.body.appendChild(stamp);
	},

	render: function(){
		// interface method
	}
};