Mario.Game = function(){
	this.scene = new Mario.Scene(Mario.Enum.RUN_GAME, this, new Mario.Input(this));
};
Mario.Game.prototype = {
	render: function(){
		// check for collisions with environment
		this.scene.actors.forEach(function(actor){
			actor.updateSpeed(this.isStanding(actor));
			do {
				// move the actors according to their speed
				var next = actor.getNextPosition();

				var bv = this.isBlockingV(actor, next);
				var bh = this.isBlockingH(actor, next);

				if (bv || bh) {
					var hitbox = actor.getHitbox();
					if (bv) {
						// cancel vert
						actor.speed.y = 0;
						if (next.y > actor.pos.y) {
							actor.pos.y = Math.ceil(actor.pos.y + hitbox.y + hitbox.h) - (hitbox.y + hitbox.h);
						} else {
							actor.pos.y = Math.floor(actor.pos.y + hitbox.y) - hitbox.y;
						}
						
						// make sure you test this after resetting the pos, isStanding depends on it
						if (actor.action == Mario.Enum.ACTION_AIRBORNE && this.isStanding(actor)) actor.updateAction(Mario.Enum.ACTION_IDLE);
					} else {
						// cancel horizontal movement
						actor.speed.x = 0;
						if (actor.action != Mario.Enum.ACTION_AIRBORNE && actor.action != Mario.Enum.ACTION_DUCKING) actor.updateAction(Mario.Enum.ACTION_IDLE);
						
						if (next.x > actor.pos.x) {
							actor.pos.x = Math.ceil(actor.pos.x + hitbox.x + hitbox.w) - (hitbox.x + hitbox.w);
						} else {
							actor.pos.x = Math.floor(actor.pos.x + hitbox.x) - hitbox.x;
						}
					}
				}
			} while (bv || bh);

			actor.pos.x = next.x;
			actor.pos.y = next.y;
			this.scene.actors.redraw = true;
		}, this);
	},
	isBlockingV: function(actor, next){
		if (next.y != actor.pos.y) {
			var hitbox = actor.getHitbox();

			var startx = Math.floor(actor.pos.x + hitbox.x); // inclusive
			var stopx = Math.ceil(actor.pos.x + hitbox.x + hitbox.w); // exclusive

			if (next.y > actor.pos.y) {
				// down, check if block down is blocking
				var y = Math.floor(next.y + hitbox.y + hitbox.h) * this.scene.map.tiles.w;
			} else {
				// up, check if block up is blocking
				var y = Math.floor(next.y + hitbox.y) * this.scene.map.tiles.w;
			}

			for (var x=startx; x<stopx; ++x) if (this.scene.grids.semantics[y+x] == Mario.Enum.TYPE_BLOCK) return true;
		}
		return false;
	},
	isBlockingH: function(actor, next, state){
		if (next.x != actor.pos.x) {
			var hitbox = actor.getHitbox();

			var starty = Math.floor(actor.pos.y + hitbox.y); // inclusive
			var stopy = Math.ceil(actor.pos.y + hitbox.y + hitbox.h); // exclusive

			if (next.x > actor.pos.x) {
				// right, check if block right is blocking
				var x = Math.floor(next.x  + hitbox.x + hitbox.w);
			} else {
				// left, check if block left is blocking
				var x = Math.floor(next.x + hitbox.x);
			}
			for (var y=starty; y<stopy; ++y) if (this.scene.grids.semantics[(y * this.scene.map.tiles.w) + x] == Mario.Enum.TYPE_BLOCK) return true;
		}
		return false;
	},
	isStanding: function(actor, test){
		var hitbox = actor.getHitbox();

		// first make sure you are in fact standing on a gridline. otherwise you cant possibly be standing on a solid.
		var y = actor.pos.y + hitbox.y + hitbox.h;

		// you're standing on a gridline if this ia round number
		if (y == (y=Math.floor(y))) {
			y *= this.scene.map.tiles.w;

			// now get the tiles below you
			var startx = Math.floor(actor.pos.x + hitbox.x); // inclusive
			var stopx = Math.ceil(actor.pos.x + hitbox.x + hitbox.w); // exclusive

			// now check for 
			for (var x=startx; x<stopx; ++x) {
				if (test) console.log(x,y/this.scene.map.tiles.w, this.scene.grids.semantics[y+x]);
				if (this.scene.grids.semantics[y+x] == Mario.Enum.TYPE_BLOCK) return true;
			}
			
			return false;
		}
	},

	getPlayer: function() {
	    return this.scene.actors[0];
    },

    setActorIdle: function() {
		this.getPlayer().updateAction(Mario.Enum.ACTION_IDLE);
    },

    jump: function() {
        var actor = this.getPlayer();
        if (!actor.speed.y && this.isStanding(actor)) {
			actor.speed.y -= Mario.Config.jumpImpulse;
			actor.updateAction(Mario.Enum.ACTION_AIRBORNE);
		}
    },
    duck: function() {
        this.getPlayer().updateAction(Mario.Enum.ACTION_DUCKING);
    },
    fire: function() {
        this.getPlayer().updateAction(Mario.Enum.ACTION_FIRE);
    },

    runLeft: function() {
		var actor = this.getPlayer();
        actor.speed.x -= Mario.Config.leftImpulse;
		actor.updateAction(Mario.Enum.ACTION_FACE_LEFT);
    },
    runRight: function() {
		var actor = this.getPlayer();
        actor.speed.x += Mario.Config.rightImpulse;
		actor.updateAction(Mario.Enum.ACTION_FACE_RIGHT);
    },

    makeSmall: function() {
        this.getPlayer().updateAction(Mario.Enum.ACTION_SMALL);
    },
    makeBig: function() {
        this.getPlayer().updateAction(Mario.Enum.ACTION_BIG);
    }
};