Mario.Scene = function(runMode, env, input) {
	this.runMode = runMode;
	this.environment = env; // should implement .render()
	this.input = input;4

	this.setMap(Mario.Map[0]);
	
	// cache the bound render function.
	this.renderBound = this.render.bind(this);

	this.contexts.background = this.layers.background.getContext('2d');
	this.layers.background.width = this.contexts.background.width = this.map.size.w;
	this.layers.background.height = this.contexts.background.height = this.map.size.h;

	if (runMode == Mario.Enum.RUN_EDIT) {
		this.contexts.types = this.layers.types.getContext('2d');
		this.layers.types.width = this.contexts.types.width = this.map.size.w;
		this.layers.types.height = this.contexts.types.height = this.map.size.h;
		
		this.layers.actors.style.display = 'none';
	} else if (runMode == Mario.Enum.RUN_GAME) {
		this.contexts.actors = this.layers.actors.getContext('2d');
		this.layers.actors.width = this.contexts.actors.width = this.map.size.w;
		this.layers.actors.height = this.contexts.actors.height = this.map.size.h;

		this.layers.types.style.display = 'none';
	}

	// start
	this.schedule();
};
Mario.Scene.prototype = {
	runMode: null,
	speed: 40, // ms interval
	renderBound: null, // this.render.bind(this)

	environment: null, // Game, Editor, should implement a "render" method
	input: null,

	frames: 0,

	layers: {
		container: document.querySelector('.layers'),
		background: document.querySelector('.layers>.background'),
		types: document.querySelector('.layers>.types'),
		actors: document.querySelector('.layers>.actors')
	},
	contexts: {
		background: null,
		types: null
	},

	map: null,
	grids: null,
	actors: null,
	
	origin: {x:0, y:0},
	
	setMap: function(map){
		this.map = map;
		this.grids = {};
		this.grids.background = map.background.slice(0);
		this.grids.background.redraw = true;
		this.grids.semantics = map.semantics.slice(0);
		this.grids.semantics.redraw = true;
		this.actors = [Mario.Actor.create.apply(Mario.Actor, map.actors)]; // TOFIX
		this.actors.redraw = true;
	},
	
	schedule: function(){
		setTimeout(this.renderBound, 1000 / Mario.Config.fps);
	},
	render: function(){
		++this.frames;
		try {
			// process input
			if (this.input) this.input.frame();
			// fire the controller
			if (this.environment) this.environment.render();

			if (this.grids.background.redraw) {
				this.contexts.background.clearRect(0,0, this.map.size.w, this.map.size.h);
				this.grids.background.forEach(this.renderBackgroundTile, this);
				this.grids.background.redraw = false;
			}
			if (this.grids.semantics.redraw) {
				if (this.runMode == Mario.Enum.RUN_EDIT) {
					this.contexts.types.clearRect(0,0, this.map.size.w, this.map.size.h);
					this.grids.semantics.forEach(this.renderTypeTile, this);
				}
				this.grids.semantics.redraw = false;
			}
			if (this.actors.redraw) {
				if (this.runMode == Mario.Enum.RUN_GAME) {
					this.contexts.actors.clearRect(0,0, this.map.size.w, this.map.size.h);
					this.actors.forEach(this.renderActor, this);
					this.actors.redraw = false;
				}
			}
		} finally {
			// regardless of what happened in the loop, always reschedule the next frame
			this.schedule();
		}
	},
	renderBackgroundTile: function(tile, i){
		if (tile--) { // if 0, no tile is set, otherwise decrease by 1 to get the actual used tile
			var sx = tile % this.map.tileSet.tiles.w;
			var sy = Math.floor(tile / this.map.tileSet.tiles.w);

			var tx = i % this.map.tiles.w;
			var ty = Math.floor(i / this.map.tiles.w);

			var tileSize = Mario.Config.tileSize;

			this.contexts.background.drawImage(
				this.map.tileSet.img,
				(this.map.tileSet.size.w+1) * sx, (this.map.tileSet.size.h+1) * sy, // sxy
				this.map.tileSet.size.w, this.map.tileSet.size.h-1, // swh
				tileSize.w * tx, tileSize.h * ty, // txy
				tileSize.w, tileSize.h // twh
			);
			
			if (Mario.Config.drawBackgroundTileCoords) { 
				this.contexts.background.fillStyle = 'white';
				this.contexts.background.font = '8px console';
				this.contexts.background.textBaseline = 'top';
				this.contexts.background.fillText(
					tx,
					tileSize.w * tx + 2, 
					tileSize.h * ty
				);
				this.contexts.background.fillText(
					ty,
					tileSize.w * tx + 2, 
					tileSize.h * ty + 8
				);
				this.fillStyle = 'black';
			}
		}
	},
	renderTypeTile: function(n, i){
		var color = false;
		switch (n){
			case Mario.Enum.TYPE_BLOCK:
				color = 'rgba(255, 0, 0, 0.5)';
				break;
			case Mario.Enum.TYPE_WATER:
				color = 'rgba(0, 0, 255, 0.5)';
				break;
			case Mario.Enum.TYPE_DEADLY:
				color = 'rgba(0, 255, 0, 0.5)';
				break;
			case Mario.Enum.TYPE_SLOW:
				color = 'rgba(255, 255, 0, 0.5)';
				break;
			case Mario.Enum.TYPE_FAST:
				color = 'rgba(255, 127, 0, 0.5)';
				break;
		}
		if (color) {
			var tw = Mario.Config.tileSize.w;
			var th = Mario.Config.tileSize.h;

			var x = i % this.map.tiles.w;
			var y = Math.floor(i/this.map.tiles.w);
			x *= tw;
			y *= th;

			this.contexts.types.fillStyle = color;
			this.contexts.types.fillRect(x, y, tw, th);
		}
	},
	renderActor: function(actor){
		// determine image to show for this actor
		actor.render(this.frames);
		var tw = this.map.size.w;
		var th = this.map.size.h;
		
		var x = actor.pos.x * Mario.Config.tileSize.w;
		var y = actor.pos.y * Mario.Config.tileSize.h;
		
		if (actor.facing == Mario.Enum.FACING_LEFT) {
			this.contexts.actors.save();
			this.contexts.actors.transform( // mirror by x
			  -1, 0,
			  0, 1,
			  this.map.size.w, 0
			);
			this.contexts.actors.drawImage(
				actor.data.spriteMap,
				actor.frame * 17,
				0,
				actor.data.width,
				actor.data.height,
				this.map.size.w-(x+16),
				y,
				16,
				32
			);
			this.contexts.actors.restore();
		} else {
			this.contexts.actors.drawImage(
				actor.data.spriteMap,
				actor.frame * 17,
				0,
				actor.data.width,
				actor.data.height,
				x,
				y,
				16,
				32
			);
		}

		if (Mario.Config.drawSpriteBox) {
			this.contexts.actors.strokeStyle = 'black';
			this.contexts.actors.strokeRect(
				actor.pos.x * tw,
				actor.pos.y * th,
				actor.data.width * tw,
				actor.data.height * th
			);
		}
		if (Mario.Config.drawHitBox) {
			var hitbox = actor.getHitbox();
			this.contexts.actors.strokeStyle = 'yellow';
			this.contexts.actors.strokeRect(
				(actor.pos.x * tw) + hitbox.x,
				(actor.pos.y * th) + hitbox.y,
				hitbox.w,
				hitbox.h
			);
		}
	},

	updateTetris: function(tetrisGame){
		// first copy the entire well. this will overwrite an entire block of cells
		this.copyWell(tetrisGame.well.blocks, tetrisGame.well.height);
		// now copy only the existing blocks of every tetrimino
		tetrisGame.tetrominoes.forEach(function(tetrimino){
			this.copyTetrimino(tetrimino, tetrisGame.well.height);
		}, this);

		this.grids.background.redraw = true;
		this.grids.semantics.redraw = true;
	},
	copyWell: function(well, height){
		var tilesw = this.map.tiles.w;
		var starty = (this.map.tiles.h - 1) - height;
		for (var i=0; i<well.length; ++i) {
			for (var j=0; j<well[i].length; ++j) {
				// yes yes, inefficient bla
				var x = j+1; // (leave the left part alone...)
				var y = starty+i; // leave the bottom part alone
				// either a block exists or it doesnt.
				var value = well[i][j] ? 1 : 0;
				
				this.grids.background[x + (y * tilesw)] = value;
				this.grids.semantics[x + (y * tilesw)] = value;
			}
		}
	},
	copyTetrimino: function(tetrimino, height){
		var starty = (this.map.tiles.h - 1) - height;
		var tw = this.map.tiles.w;
		var x = tetrimino.x;
		var y = tetrimino.y + starty;
		
		for (var i=0; i<tetrimino.blocks.length; ++i) {
			for (var j=0; j<tetrimino.blocks[i].length; ++j) {
				if (tetrimino.blocks[i][j]) {
					this.grids.background[x + j + 1 + + ((y + i - 1) * tw)] = 1;
					this.grids.semantics[x + j + ((y + i - 1) * tw)] = 1;
				}
			}
		}
	},
0:0};